package cz.muni.fi.pv243.geocaching.caches;

import cz.muni.fi.pv243.geocaching.caches.crud.CacheManager;
import cz.muni.fi.pv243.geocaching.caches.logs.LogCacheManager;
import cz.muni.fi.pv243.geocaching.model.enums.CacheStatus;
import cz.muni.fi.pv243.geocaching.model.enums.LogCacheType;
import cz.muni.fi.pv243.geocaching.session.LoggedInUserBean;
import cz.muni.fi.pv243.geocaching.model.*;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.inject.Inject;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author oja
 */
@ManagedBean(name="cacheManagedBean")
@ViewScoped
public class CacheManagedBean implements Serializable {

    @Inject
    private CacheManager cm;
    
    @Inject
    private LogCacheManager lcm;
    
    @Inject
    private LoggedInUserBean loggedBean;
    
    private Cache cache;
    private String cacheCode;
    
    private List<LogCache> logs;
    private int foundCount = 0;
        
    public void initialize(ComponentSystemEvent event) {
        cacheCode = FacesContext.getCurrentInstance().getExternalContext()
                .getRequestParameterMap().get("cacheid");
        if (cacheCode!=null) {  
            try {
                cache = cm.find(new Integer(cacheCode));
            } catch (NumberFormatException e) {
                cache = null;
            }
            if (cache!=null) {
                logs = new ArrayList<LogCache>();
                Iterator it = lcm.findAll().iterator();
                while (it.hasNext()) {
                    LogCache log = (LogCache)it.next();
                    if (log.getCache().equals(cache)) {
                        logs.add(log);
                        if (log.getLogCacheType()== LogCacheType.VISITED) {
                            foundCount++;
                        }
                    }                    
                }       
            }            
        }
    }
    
    public boolean canChangeStatus() {
        if (loggedBean.getLoggedInUser()==null) {
            return false;
        } else { 
            return FacesContext.getCurrentInstance().getExternalContext().isUserInRole("REVIEWER")
                    || FacesContext.getCurrentInstance().getExternalContext().isUserInRole("ADMIN")
                    || cache.getOwner().getUsername().equals(loggedBean.getLoggedInUser().getUsername());
        }
    }    
    
    public boolean canBeArchived() {
        return canChangeStatus() && cache.getCacheStatus()!= CacheStatus.ARCHIVED;
    }
    
    public boolean canBeActiveAgain() {
        return canChangeStatus() && cache.getCacheStatus()==CacheStatus.ARCHIVED;
    }
    
    public boolean canBeDisabled() {
        return canChangeStatus() && cache.getCacheStatus()==CacheStatus.ACTIVE;
    }
    
    public boolean canBeEnabled() {
        return canChangeStatus() && cache.getCacheStatus()==CacheStatus.DISABLED;
    }
    
    public void archiveCache() throws IOException {
        if (canChangeStatus()) {
            cache.setCacheStatus(CacheStatus.ARCHIVED);
            cm.edit(cache);
        }
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public void activeAgainCache () throws IOException {
        if (canChangeStatus() && cache.getCacheStatus()!=CacheStatus.NEW) {
            cache.setCacheStatus(CacheStatus.ACTIVE);
            cm.edit(cache);
        }
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public void disableCache() throws IOException {
        if (canChangeStatus()) {
            cache.setCacheStatus(CacheStatus.DISABLED);
            cm.edit(cache);
        }
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public void enableCache() throws IOException {
        if (canChangeStatus() && cache.getCacheStatus()!=CacheStatus.NEW) {
            cache.setCacheStatus(CacheStatus.ACTIVE);
            cm.edit(cache);
        }
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public boolean canBePublished() {
        if (loggedBean.getLoggedInUser()==null) {
            return false;
        } else { 
            return FacesContext.getCurrentInstance().getExternalContext().isUserInRole("REVIEWER")
                     && cache.getCacheStatus()==CacheStatus.NEW;
        }
    }
    
    public void publishCache() throws IOException {
        cache.setCacheStatus(CacheStatus.ACTIVE);
        cm.edit(cache);
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/reviewer/publishcache.xhtml");
    }
    
    public void processLogCache() throws IOException {
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/logcache.xhtml?cacheid="+cache.getId());
    }
    
    public boolean canEditCache() {
        if (loggedBean.getLoggedInUser()==null) {
            return false;
        } else { 
            return FacesContext.getCurrentInstance().getExternalContext().isUserInRole("ADMIN")
                    || cache.getOwner().getUsername().equals(loggedBean.getLoggedInUser().getUsername());
        }
    }
    
    public void processEditCache() throws IOException {
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/editcache.xhtml?cacheid="+cache.getId());
    }
    
    public void deleteLog(LogCache log) throws IOException {
        lcm.remove(log);
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public boolean canBeRendered() {
        if (loggedBean.getLoggedInUser()==null || cache==null) {
            return false;
        } else {
            if (cache.getCacheStatus()==CacheStatus.NEW) {
                return FacesContext.getCurrentInstance().getExternalContext().isUserInRole("REVIEWER")
                    || FacesContext.getCurrentInstance().getExternalContext().isUserInRole("ADMIN")
                    || cache.getOwner().getUsername().equals(loggedBean.getLoggedInUser().getUsername());
            } else {
                return true;
            }
        }
        
    }
    
    public boolean cacheExists () {
        return cache!=null;        
    }
    
    public boolean logCacheExists () {
        return logs!=null;        
    }

    public Cache getCache() {
        return cache;
    }

    public String getCacheCode() {
        return cacheCode;
    }

    public void setCacheCode(String cacheCode) {
        this.cacheCode = cacheCode;
    }

    public List<LogCache> getLogs() {
        return logs;
    }

    public void setLogs(List<LogCache> logs) {
        this.logs = logs;
    }

    public int getFoundCount() {
        return foundCount;
    }

    public void setFoundCount(int foundCount) {
        this.foundCount = foundCount;
    }
    
}
