/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pv243.geocaching.caches.crud;

import cz.muni.fi.pv243.geocaching.model.enums.CacheType;
import cz.muni.fi.pv243.geocaching.session.LoggedInUserBean;
import cz.muni.fi.pv243.geocaching.model.*;
import java.io.IOException;
import java.io.Serializable;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.ComponentSystemEvent;
import javax.inject.Inject;

/**
 *
 * @author oja
 */
@ManagedBean(name = "editCacheManagedBean")
@ViewScoped
public class EditCacheManagedBean implements Serializable {
    
    @Inject
    private CacheManager cm;
    
    @Inject
    private LoggedInUserBean loggedBean;
    
    private Cache cache;
    private String cacheCode;
    
    private String name;
    private int type;
    private int coorx;
    private int coory;
    private int difficulty;
    private int terrain;
    private String listing;  

    public void initialize(ComponentSystemEvent event) {
        cacheCode = FacesContext.getCurrentInstance().getExternalContext()
                .getRequestParameterMap().get("cacheid");
        if (cacheCode!=null) {          
            try {
                cache = cm.find(new Integer(cacheCode)); 
            } catch (NumberFormatException e) {
                cache = null;
            }                              
            if (cache!=null) {
                name = cache.getName();
                coorx = cache.getCoor().getX();
                coory = cache.getCoor().getY();
                difficulty = cache.getDifficulty();
                terrain = cache.getTerrain();
                listing = cache.getListing();
                if (cache.getCacheType()== CacheType.TRADITIONAL) type = 1;
                if (cache.getCacheType()==CacheType.MULTI) type = 2;
                if (cache.getCacheType()==CacheType.MYSTERY) type = 3;
                if (cache.getCacheType()==CacheType.EARTH) type = 4;
                if (cache.getCacheType()==CacheType.LETTERBOX) type = 5;
            }
        }
    }    
    
    public void editCache() throws IOException {        
        CacheType cacheType;
        switch(type) {
            case 1: cacheType = CacheType.TRADITIONAL; break;
            case 2: cacheType = CacheType.MULTI; break;
            case 3: cacheType = CacheType.MYSTERY; break;
            case 4: cacheType = CacheType.EARTH; break;
            case 5: cacheType = CacheType.LETTERBOX; break;
            default: cacheType = CacheType.TRADITIONAL; break;
        }             
        cache.setName(name);
        cache.setCacheType(cacheType);
        cache.setCoor(new Coordinates(coorx,coory));
        cache.setDifficulty(difficulty);
        cache.setTerrain(terrain);       
        cache.setListing(listing); 
        
        cm.edit(cache);
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public void processBack() throws IOException {
        ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
        externalContext.redirect("/cache.xhtml?cacheid="+cache.getId());
    }
    
    public boolean canBeRendered() {
        if (loggedBean.getLoggedInUser()==null || cache==null) {
            return false;
        } else {
            return FacesContext.getCurrentInstance().getExternalContext().isUserInRole("ADMIN")
                    || cache.getOwner().getUsername().equals(loggedBean.getLoggedInUser().getUsername());            
        }       
        
    }
    
    public boolean cacheExists () {
        return cache!=null;        
    }

    public Cache getCache() {
        return cache;
    }

    public void setCache(Cache cache) {
        this.cache = cache;
    }

    public String getCacheCode() {
        return cacheCode;
    }

    public void setCacheCode(String cacheCode) {
        this.cacheCode = cacheCode;
    }

    public int getCoorx() {
        return coorx;
    }

    public void setCoorx(int coorx) {
        this.coorx = coorx;
    }

    public int getCoory() {
        return coory;
    }

    public void setCoory(int coory) {
        this.coory = coory;
    }

    public int getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(int difficulty) {
        this.difficulty = difficulty;
    }

    public String getListing() {
        return listing;
    }

    public void setListing(String listing) {
        this.listing = listing;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getTerrain() {
        return terrain;
    }

    public void setTerrain(int terrain) {
        this.terrain = terrain;
    }

    public int getType() {
        return type;
    }

    public void setType(int type) {
        this.type = type;
    }
    
    
}
