package cz.muni.fi.pv243.geocaching.model;

import cz.muni.fi.pv243.geocaching.model.enums.CacheStatus;
import cz.muni.fi.pv243.geocaching.model.enums.CacheType;
import org.slf4j.Logger;

import javax.inject.Inject;
import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

/**
 *
 * @author oja
 */
@Entity
@Table(name="Cache")
public class Cache implements Serializable {
    
    @Inject
    @Transient
    private Logger logger;
    
    @Id
    @GeneratedValue(strategy=GenerationType.SEQUENCE)
    private Integer id;
    
    private String name;
    private CacheType cacheType;
    private Coordinates coor;
    private int difficulty;
    private int terrain;
    @ManyToOne
    private User owner;
    @Temporal(javax.persistence.TemporalType.DATE)
    private Date hidden;
    private String listing;
    private CacheStatus cacheStatus = CacheStatus.NEW;
    
    @PostPersist    
    public void logCreation() {
        logger.info("Created new cache: " + this.toString());
    }

    public Cache() {        
    }    

    public Cache(String name, CacheType cacheType, Coordinates coor, int difficulty, int terrain, User owner, Date hidden, String listing) {
        this.name = name;
        this.cacheType = cacheType;
        this.coor = coor;
        this.difficulty = difficulty;
        this.terrain = terrain;
        this.owner = owner;
        this.hidden = hidden;
        this.listing = listing;
        cacheStatus = CacheStatus.NEW;
    }

    public CacheType getCacheType() {
        return cacheType;
    }

    public void setCacheType(CacheType cacheType) {
        this.cacheType = cacheType;
    }

    public Coordinates getCoor() {
        return coor;
    }

    public void setCoor(Coordinates coor) {
        this.coor = coor;
    }

    public int getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(int difficulty) {
        this.difficulty = difficulty;
    }

    public Date getHidden() {
        return hidden;
    }

    public void setHidden(Date hidden) {
        this.hidden = hidden;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getListing() {
        return listing;
    }

    public void setListing(String listing) {
        this.listing = listing;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public User getOwner() {
        return owner;
    }

    public void setOwner(User owner) {
        this.owner = owner;
    }

    public int getTerrain() {
        return terrain;
    }

    public void setTerrain(int terrain) {
        this.terrain = terrain;
    }

    public CacheStatus getCacheStatus() {
        return cacheStatus;
    }

    public void setCacheStatus(CacheStatus cacheStatus) {
        this.cacheStatus = cacheStatus;
    }

    public boolean equals(Cache other) {
        return id.equals(other.id);
    }
    
    @Override
    public String toString() {
        return "Cache{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", cacheType=" + cacheType +
                ", coor=" + coor +
                ", difficulty=" + difficulty +
                ", terrain=" + terrain +
                ", owner=" + owner.getUsername() +
                ", hidden=" + hidden +
                ", listing='" + listing + '\'' +
                ", cacheStatus=" + cacheStatus +
                '}';
    }



}
