package cz.muni.fi.pv243.geocaching.model;

import cz.muni.fi.pv243.geocaching.model.enums.Roles;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;

import javax.faces.validator.FacesValidator;
import javax.inject.Inject;
import javax.persistence.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;
import java.util.TreeSet;

/**
 * Author: jmartisk
 * Date: 4/18/12
 * Time: 7:03 PM
 */
@Entity
@Table(name="User")
public class User {

    public static final int MINIMUM_PASSWORD_LENGTH = 5;

    @Inject
    @Transient
    private Logger logger;

    // TODO: jak zajistit integritni omezeni ciziho klice z tabulky UserRoles na username ??
    @ElementCollection(fetch = FetchType.EAGER)
    @Enumerated(EnumType.STRING)
    @Column(name="role", nullable = false)
    @JoinTable(name = "UserRoles", joinColumns = @JoinColumn(name = "User_username"))
    private Collection<Roles> roles = new TreeSet<Roles>();

    @Id
    private String username;
    public static final int MAXIMUM_PASSWORD_LENGTH = 32;

    public String getPassword() {
        return password;
    }

    /**
     * SHA-256 hash of the password encoded in Base64
     */
    @NotNull
    @Size(min = 44, max = 44)
    private String password;

    @PostPersist
    public void logCreation() {
        logger.info("Registered new user: " + this.toString());
    }

    public boolean isUserInRole(Roles role) {
        return roles.contains(role);
    }

    public boolean isReviewer() {
        return roles.contains(Roles.REVIEWER);
    }

    public User() {
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setPassword(String password) {
        if((password==null)||(password.length() < MINIMUM_PASSWORD_LENGTH))
            throw new SecurityException("Password of minimal length " + MINIMUM_PASSWORD_LENGTH + " is required");
        this.password = hashPassword(password);
    }

    public void addRole(Roles role) {
        roles.add(role);
    }

    public static String hashPassword(String password) {
        try {
            MessageDigest m = MessageDigest.getInstance("SHA-256");
            m.update(password.getBytes(), 0, password.length());
            return Base64.encodeBase64String(m.digest());
        } catch (NoSuchAlgorithmException ex) {
            throw new Error(ex);
        }
    }

    @Override
    public String toString() {
        return "User{" +
                "roles=" + roles +
                ", username='" + username + '\'' +
                ", password='" + password + '\'' +
                '}';
    }
}
