package cz.muni.fi.pv243.geocaching.session;

import cz.muni.fi.pv243.geocaching.model.User;
import org.slf4j.Logger;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.servlet.http.HttpSession;
import java.io.Serializable;
import java.security.Principal;

/**
 * Author: jmartisk
 * Date: 6/8/12
 * Time: 8:50 AM
 */
@SessionScoped
@ManagedBean
public class LoggedInUserBean implements Serializable {

    @Inject
    private Logger logger;

    @PersistenceContext(unitName = "geocachingPU")
    private EntityManager em;

    public LoggedInUserBean() {
        userLoggedIn = false;
    }

    @PostConstruct
    public void postConstruct() {
        refreshLoginContext();
        if (loggedInUser != null) {
            logger.info("Created LoggedInUserBean, user="+loggedInUser.getUsername());
        } else {
            logger.info("Created LoggedInUserBean, no user logged in");
        }
    }

    private boolean userLoggedIn = false;
    private User loggedInUser = null;

    public User getLoggedInUser() {
        refreshLoginContext();
        return loggedInUser;
    }

    public boolean isUserLoggedIn() {
        refreshLoginContext();
        return userLoggedIn;
    }

    public void refreshLoginContext() {
        ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
        Principal principal = ctx.getUserPrincipal();
        HttpSession session = (HttpSession)ctx.getSession(true);
        boolean isNew = !userLoggedIn;
        if ((principal != null ) && (!principal.getName().equals("anonymous"))) {
            loggedInUser = em.find(User.class, principal.getName());
            if (loggedInUser != null) {
                if(isNew)
                    logger.info("Associated HttpSession " + session.getId() + " with username="+loggedInUser.getUsername());
                session.setAttribute("username", loggedInUser.getUsername());
                userLoggedIn = true;
            } else {
                throw new SecurityException("Authenticated principal " + principal.getName() + " does not match any user in the database");
            }
        } else {
            userLoggedIn = false;
            loggedInUser = null;
        }
    }

    public String logout() {
        logger.info("Logout button clicked, invalidating session");
        ExternalContext ctx = FacesContext.getCurrentInstance().getExternalContext();
        ctx.invalidateSession();
        userLoggedIn = false;
        loggedInUser = null;
        return "logout";
    }

}
